unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Menus, IniFiles, IdBaseComponent,
  IdComponent, IdUDPBase, IdUDPClient, IdAntiFreezeBase, IdAntiFreeze,
  ComCtrls, IdSNTP;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Panel1: TPanel;
    Edit1: TEdit;
    IdUDPClient1: TIdUDPClient;
    IdAntiFreeze1: TIdAntiFreeze;
    StatusBar1: TStatusBar;
    Memo1: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    Edit2: TEdit;
    UDP1: TMenuItem;
    Clear1: TMenuItem;
    SNTP1: TMenuItem;
    RequestSNTP1: TMenuItem;
    IdSNTP1: TIdSNTP;
    CommandA1: TMenuItem;
    CommandB1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Edit1Exit(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
    procedure Edit2KeyPress(Sender: TObject; var Key: Char);
    procedure Edit2Exit(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure CommandA1Click(Sender: TObject);
    procedure CommandB1Click(Sender: TObject);
  private
    { Private declarations }
    ReceiveBuffer: string;
    TransmitBuffer: string;
    SNTP_RxBuffer: array[0..128] of byte;
    SNTP_TxBuffer: array[0..128] of byte;
    procedure Communicate;
  public
    { Public declarations }
    function GetPath: string;
    procedure CheckForIni;
    procedure GetIni;
    procedure SetIni;
    function IsIPAddress(zvIPAddressID: string): boolean;
    function IsStrANumber(S: string): Boolean;

    function GetFirstToken(S: string; Token: Char): string;
    function StripFirstToken(S: string; Token: Char): string;
    function Int2StrPad0(N: LongInt; Len: Integer): string;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
const
  CmdTimeout = 10000;

  NUL = #0;   //null
  SOH = #1;   //start of heading
  STX = #2;   //start of text
  ETX = #3;   //end of text
  EOT = #4;   //end of transmission
  ENQ = #5;   //enquiry
  ACK = #6;   //acknowledge
  BEL = #7;   //bell
  BS  = #8;   //backspace
  HT  = #9;   //horizontal tab
  LF  = #10;  //line feed
  VT  = #11;  //vertical tab
  FF  = #12;  //form feed
  CR  = #13;  //carriage return
  SO  = #14;  //shift out
  SI  = #15;  //shift in
  DLE = #16;  //data link escape
  DC1 = #17;  //device control 1
  DC2 = #18;  //device control 2
  DC3 = #19;  //device control 3
  DC4 = #20;  //device control 4
  NAK = #21;  //negative acknowledge
  SYN = #22;  //synchronous idle
  ETB = #23;  //end of trans. block
  CAN = #24;  //cancel
  EM  = #25;  //end of medium
  SUB = #26;  //substitute
  ESC = #27;  //escape
  FS  = #28;  //file separator
  GS  = #29;  //group separator
  RS  = #30;  //record separator
  US  = #31;  //unit separator
  DEL = #127; //delete

  CRLF = #13+#10;


{Get current directory}
function TForm1.GetPath: string;
var
  Path: string;
  Index: integer;
begin
  Path := ParamStr(0);
  Index := LastDelimiter('\',Path);
  Result := Copy(Path,1,Index);
end;

{Check if ini file already exists. If not, create it}
procedure TForm1.CheckForIni;
var
  F: System.Text;
  Path: string;
begin
  Path := GetPath + 'UDP_Utility.ini';
  if not FileExists(Path) then
  begin
    System.Assign(F,Path);
    System.ReWrite(F);
    System.WriteLn(F,'[IP_Address]');
    System.WriteLn(F,'CurAddr=192.168.2.4');
    System.WriteLn(F,'[Port]');
    System.WriteLn(F,'CurPort=6060');
    System.Close(F);
  end;
end;

{Read configuration from ini file}
procedure TForm1.GetIni;  {uses IniFiles}
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(GetPath + 'UDP_Utility.ini');
  Edit1.Text := Ini.ReadString('IP_Address','CurAddr','192.168.2.4');
  Edit2.Text := Ini.ReadString('Port','CurPort','6060');
  Ini.Free;
end;

{Update ini file}
procedure TForm1.SetIni;  {uses IniFiles}
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(GetPath + 'UDP_Utility.ini');
  Ini.WriteString('IP_Address', 'CurAddr', Edit1.Text);
  Ini.WriteString('Port','CurPort',Edit2.Text);
  Ini.Free;
end;

{This function return true if the passed string is a true IP address.
 From Internet at http://www.delphi3000.com/articles/article_3458.asp?SK=}
function TForm1.IsIPAddress(zvIPAddressID: string): boolean;
var 
  ipLengthOfGroup, ipPosition, ipNoOfGroups: integer; 
  ipGroupNum, Code: integer; 
begin 
  IsIpAddress := False;
  ipNoOfGroups := 0;
  ipLengthOfGroup := 0;
  for ipPosition := 1 to Length(zvIPAddressID) do
    case Ord(zvIPAddressID[ipPosition]) of
      48..57: 
        begin 
          inc(ipLengthOfGroup); 
          if (ipLengthOfGroup > 3) then exit; 
        end; 
      46: 
        begin 
          inc(ipNoOfGroups); 
          Val(Copy(zvIPAddressID, ipPosition - ipLengthOfGroup, ipLengthOfGroup), ipGroupNum, Code); 
          if ((ipNoOfGroups > 3) or (ipLengthOfGroup = 0)) or (ipGroupNum > 255) then exit; 
          ipLengthOfGroup := 0; 
        end; 
      else 
        exit; 
    end; 
  Val(Copy(zvIPAddressID, ipPosition - ipLengthOfGroup, ipLengthOfGroup), ipGroupNum, Code);
  IsIPAddress := (ipNoOfGroups = 3) and (ipLengthOfGroup > 0) and (ipGroupNum < 256);
end;


function TForm1.IsStrANumber(S: string): Boolean;
var
  P: PChar;
begin
  P      := PChar(S);
  Result := False;
  while P^ <> #0 do
  begin
    if not (P^ in ['0'..'9']) then Exit;
    Inc(P);
  end;
  Result := True;
end;


function TForm1.GetFirstToken(S: string; Token: Char): string;
var
  Count: Integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count < 1 then
  begin
    GetFirstToken := '';
    Exit;
  end;
  Dec(Count);
  Temp := Copy(S,1,Count);
  GetFirstToken := Temp;
end;

function TForm1.StripFirstToken(S: string; Token: Char): string;
var
  Count: integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count = 0 then begin
    StripFirstToken := S;
    Exit;
  end;
  Temp := S;
  Delete(Temp,1,Count);
  StripFirstToken := Temp;
end;

function TForm1.Int2StrPad0(N: LongInt; Len: Integer): string;
var
  S : string;
begin
  Str(N:0,S);
  while Length(S) < Len do
    S := '0' + S;
  Int2StrPad0 := S;
end;

procedure TForm1.Communicate;
begin
  IdUDPClient1.ReceiveTimeout := CmdTimeout;
  IdUDPClient1.Send(TransmitBuffer);
  ReceiveBuffer := IdUDPClient1.ReceiveString();
  if ReceiveBuffer = '' then
  begin
    Memo1.Lines.Add('No response received from the server after ' + IntToStr(IdUDPClient1.ReceiveTimeout div 1000) + ' seconds.');
  end
  else
  begin
    ReceiveBuffer := Trim(ReceiveBuffer);
    Memo1.Lines.Add(ReceiveBuffer);
  end;
end;

//******************************************************************************
procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CheckForIni;
  GetIni;
  Panel1.Caption := '';
  Memo1.Clear;

  IdUDPClient1.Active := True;
  IdUDPClient1.Host := Edit1.Text;
  IdUDPClient1.Port := StrToInt(Edit2.Text);

  IdSNTP1.Active := True;
  IdSNTP1.Host := Edit1.Text;
end;

procedure TForm1.Edit1Exit(Sender: TObject);
begin
  if IsIPAddress(Edit1.Text) then  // Should have a valid IP address here
  begin
    IdUDPClient1.Host := Edit1.Text;  // Set host IP address from box
    IdSNTP1.Host := Edit1.Text;
    StatusBar1.Panels[0].Text := '';
  end
  else
  begin
    StatusBar1.Font.Color := clRed;
    StatusBar1.Panels[0].Text := 'Invalid IP address';
  end;
end;

procedure TForm1.Clear1Click(Sender: TObject);
begin
  Memo1.Clear;
end;

procedure TForm1.Edit1KeyPress(Sender: TObject; var Key: Char);
begin
  if key = Chr(VK_RETURN) then
     Edit2.SetFocus;
end;

procedure TForm1.Edit2KeyPress(Sender: TObject; var Key: Char);
begin
  if key = Chr(VK_RETURN) then
     Edit1.SetFocus;
end;

procedure TForm1.Edit2Exit(Sender: TObject);
begin
  if IsStrANumber(Edit2.Text) = True then  // Should have a valid port here
  begin
    IdUDPClient1.Port := StrToInt(Edit2.Text);  // Set port
    StatusBar1.Panels[0].Text := '';
  end
  else
  begin
    StatusBar1.Font.Color := clRed;
    StatusBar1.Panels[0].Text := 'Invalid Port';
  end;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if (IsIPAddress(Edit1.Text) AND IsStrANumber(Edit2.Text)) then
  begin
      SetIni;
      CanClose := true;
  end
  else
      StatusBar1.Panels[0].Text := 'Invalid IP or Port';
end;

procedure TForm1.CommandA1Click(Sender: TObject);
begin
  TransmitBuffer := 'A'+CRLF;
  Communicate;
end;

procedure TForm1.CommandB1Click(Sender: TObject);
begin
  TransmitBuffer := 'B'+CRLF;
  Communicate;
end;

end.
