/**************************************************************************************************
  USB_Driver
  This module is used for communications with the user through a custom PC program.
**************************************************************************************************/
#include "USB_Driver.h"
#include "Buffers.h"
//#include "Common.h"
#include "__Time.h"
#include "WiFiPlus.h"
#include "UART0.h"
#include "IO.h"


// Variables
char readbuff[64];
char writebuff[64];

tx_q0_struct txqueue0;        // Transmit buffer FIFO and input/output pointers

extern TimeStruct ts1;
extern long Epoch;
extern unsigned int RxState;
extern char TxMsgLength;
extern char myAP_Key[36];
extern char DataBuffer[128];

const code char TestArray[] = {0x41,0x42,0x0D,0x0A,0x00};

char OutputBuf[64];

//char myIpAdd[4] = {192,168,1,110};

/*
struct {
     char   macAddress[6];      // My MAC address
     char   ipAddress[4];       // My IP address
     char   networkMask[4];     // Network mask (for example : 255.255.255.0)
     char   gatewayAddress[4];  // Gateway (router) IP address
     char   networkStatus;      // Network status
} Net_Wireless_MCW1001_Properties;
*/

char IP_Addr0;
char IP_Addr1;
char IP_Addr2;
char IP_Addr3;

// Defined in IO.c
//sbit Net_Wireless_MCW1001_Chip_RST at GPIO_PORTC_DATA.B5;
//sbit Net_Wireless_MCW1001_Chip_RST_Direction at GPIO_PORTC_DIR.B5;


extern tPointerFcn WiFiStateFuncPtr;
extern char DataBuffer[128];
extern char myIpAdd[4];

/**************************************************************************************************
  This is the USB interrupt handler */

void USB0_Interrupt() iv IVT_INT_USB0 ics ICS_AUTO {
     USB_Interrupt_Proc();
}

/**************************************************************************************************
  Initialization */
void USB_Init(void) {
     //USBIE_bit = 0;
     //IPC11bits.USBIP = 7;
   
     HID_Enable(&readbuff,&writebuff);
     USB_TxQueueInit();
}

/**************************************************************************************************
  This routine initializes the transmit buffer FIFO in and out pointers */
void USB_TxQueueInit(void) {
     txqueue0.in = 0;
     txqueue0.out = 0;
}

/**************************************************************************************************
  This routine makes an entry into the transmit FIFO and updates the in pointer to the next
  available location. */
void USB_PutQueue(char *msg) {
     txqueue0.q[txqueue0.in++] = msg;
     if (txqueue0.in > (txqsize0-1))
         txqueue0.in = 0;
}

/**************************************************************************************************
  This routine checks the transmit buffer FIFO and, if it is not empty, sends the buffer to USB.
  If the buffer was successfully sent, the out pointer is updated to the next available location.
  */

// Call this function in main loop
void USB_CheckOut(void) {
     //char *USB_XmtBuf;
     
     if (txqueue0.in != txqueue0.out) {          // Look for a pointer in the FIFO
          
          //USB_XmtBuf = txqueue0.q[txqueue0.out];
          
          if (HID_Write(txqueue0.q[txqueue0.out],64) != 0) {
              txqueue0.out++;
              if (txqueue0.out > (txqsize0-1))   // If past the end of array
                  txqueue0.out = 0;              // set pointer to start of array
/*
              if (USB_XmtBuf == Buffer1) {       // Only do this to Buffer1
                 Buffer1[0] = 0;    // Invalidate the buffer (single buffer overrun problem)
              }
*/
          }
     }
}

/**************************************************************************************************
  This routine checks USB for a receive message then decodes the first character for a command. */

// Call this function in main loop
void USB_CheckIn(void) {
     char *BufNum;
     int Year, Month, Day, DoW, Hour, Minute, Second;
     char String[8];

     if (HID_Read() != 0) {
          switch (readbuff[0]) {
               
               case 'a':  // Show date/time
                   Year = ts1.yy;    // MikroE sprintf has trouble with char type
                   Month = ts1.mo;
                   Day = ts1.md;
                   Hour = ts1.hh;
                   Minute = ts1.mn;
                   Second = ts1.ss;

                   BufNum = GetBuffer();     // Get a free buffer
                   sprintf(BufNum,"%04d/%02d/%02d %02d:%02d:%02d\r\n",Year,Month,Day,Hour,Minute,Second);
                   USB_PutQueue(BufNum);
                   break;
               
               case 'b':  // Set clock/calendar from data in message
               //  byte: 0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22
               //  char: b  , YR YR  , MO MO  , MD MD  , WD WD  ,  h  h  ,  m  m  ,  s  s \n
                   
                   if (strlen(readbuff) == 24) {
                       USB_PutQueue("Setting date/time \r\n");
                       String[2] = 0;

                       String[0] = readbuff[2];
                       String[1] = readbuff[3];
                       ts1.yy = 2000 + atoi(String);   // Year

                       String[0] = readbuff[5];
                       String[1] = readbuff[6];
                       ts1.mo = atoi(String);          // Month

                       String[0] = readbuff[8];
                       String[1] = readbuff[9];
                       ts1.md = atoi(String);          // Day of month

                       String[0] = readbuff[11];
                       String[1] = readbuff[12];
                       DoW = atoi(String);             // Day of week

                       switch (DoW) {                  // Translate between Delphi7 and mikroC
                           case 1: ts1.wd = 6; break;  // Sunday
                           case 2: ts1.wd = 0; break;  // Monday
                           case 3: ts1.wd = 1; break;  // Tuesday
                           case 4: ts1.wd = 2; break;  // Wednesday
                           case 5: ts1.wd = 3; break;  // Thursday
                           case 6: ts1.wd = 4; break;  // Friday
                           case 7: ts1.wd = 5; break;  // Saturday
                       }

                       String[0] = readbuff[14];
                       String[1] = readbuff[15];
                       ts1.hh = atoi(String);          // Hours

                       String[0] = readbuff[17];
                       String[1] = readbuff[18];
                       ts1.mn = atoi(String);          // Minutes

                       String[0] = readbuff[20];
                       String[1] = readbuff[21];
                       ts1.ss = atoi(String);          // Seconds

                       Epoch = Time_dateToEpoch(&ts1);
                   }
                   else {
                       BufNum = GetBuffer();     // Get a free buffer
                       sprintf(BufNum,"Incorrect length: %d \r\n",strlen(readbuff));
                       USB_PutQueue(BufNum);   // Display the buffer
                   }
                   break;
               
               case 'c':
                   WiFiPlusInit();     // Initialize
                   break;
               case 'd':
                   WiFiSwReset();      // Software reset
                   break;
               case 'e':
                   WiFiPingSend();     // Send ping
                   break;
               case 'f':
                   WiFiGetVersion();   // Get version
                   break;
               case 'g':
                   WiFiGetWpaKey();    // Get WPA key
                   break;
               case 'h':
                   break;
               case 'i':
                   break;
               case 'j':
                   break;
               case 'k':
                   WiFiStateFuncPtr = WiFiState80; // UDP server
                   break;
               case 'l':
                   WiFiStateFuncPtr = WiFiState74; // Receive from socket
                   break;
               case 'm':
                   break;
               case 'n':
                   break;
               case 'o':
                   break;
               case 'p':
                   break;
               case 'q':
                   break;
               case 'r':
                   WiFiStateFuncPtr = WiFiState55; // Connect a socket
                   break;
               case 's':
                   WiFiStateFuncPtr = WiFiState50; // Bind a socket
                   break;
               case 't':
                   WiFiStateFuncPtr = WiFiState45; // Close a socket
                   break;
               case 'u':
                   WiFiStateFuncPtr = WiFiState40; // Create UDP socket
                   break;
               case 'v':
                   WiFiStateFuncPtr = WiFiState35; // Disconnect from access point
                   break;
               case 'w':
                   WiFiStateFuncPtr = WiFiState30; // Connect to access point
                   break;
               case 'x':
                   WiFiStateFuncPtr = WiFiState15; // Set Wi-Fi parameters
                   break;
               case 'y':
                   WiFiStateFuncPtr = WiFiState4;  // Scan for access points
                   break;
               case 'z':
                   WiFiStateFuncPtr = WiFiState1;  // Get network status
                   break;
               
                   // WiFiSetArpTime();
               
               
               // Etc.
              
               default:
                   USB_PutQueue("Unknown command\r\n");
                   
                   BufNum = GetBuffer();         // Get a free buffer
                   (void)sprintf(BufNum,"%c\r\n",readbuff[0]);
                   USB_PutQueue(BufNum);
                   break;
          }
     }
}