/**************************************************************************************************
  WiFiPlus.c
  Custom WiFi code for the Microchip MCW1001A chip. Due to the asynchronous nature of this
  protocol, the code relies heavily on semaphores (flags) making it hard to follow.

  Little-Endian? Really? WHAT WERE THEY THINKING?!!!
**************************************************************************************************/

// Include Files
#include "WiFiPlus.h"
#include "Buffers.h"
#include "Common.h"
#include "UART0.h"
#include "USB_Driver.h"
#include "Timer0.h"
#include "WiFiTEst_objects.h"
#include "WiFiTEst_resources.h"

// Defines - see WiFiPlus.h


// Variables
#if USE_DHCP == TRUE
char myIpAdd[4]     = {0,0,0,0};
char myNetMask[4]   = {0,0,0,0};
char gatewayAdd[4]  = {0,0,0,0};
#else
char myIpAdd[4]     = {192,168,1,15};
char myNetMask[4]   = {255,255,255,0};
char gatewayAdd[4]  = {192,168,1,1};
#endif
char myMacAdd[6]    = {0x00,0x14,0xA5,0x76,0x19,0x3F};
char myProfileID    = 1;
char mySSID[32]     = "MySSID";        // Enter your own access point SSID
char myPassword[20] = "MyPassword";    // Enter your own password
char myAP_Key[32];


char UDP_RemoteHandle;
char UDP_RemoteIP[4] = {10,0,1,3};
unsigned int UDP_RemPort;          // Remote client will supply the working port
unsigned int UDP_LocPort = 6060;   // Our UDP server will listen on this port
unsigned int UDP_RemByteCount;
char UDP_Buf[128];                 // Buffer for incoming UDP data

char TCP_ServerSocketCount = 0;    // Use the following to partition socket buffers (see manual)
char TCP_ClientSocketCount = 0;
unsigned int TCP_ServerRcvBufSize = 0;
unsigned int TCP_ServerXmtBufSize = 0;
unsigned int TCP_ClientRcvBufSize = 0;
unsigned int TCP_ClientXmtBufSize = 0;


unsigned short int WiFiTimerHandle;         // Timer handle for the state machine
unsigned short int NumScanRslts;            // Number of scan results returned by the scan command
unsigned short int SocketHandle;            // The UDP socket handle
unsigned short int HandleValid = FALSE;     // Will be TRUE when a socket is created
unsigned short int BindResult;              // 0 - success, 1-255 - unsuccessful bind
unsigned short int ConnectResult;           // 0 -success, 254 - connection in progress, 255 - connection failed
unsigned short int BacklogCount;            // Result of SOCKET_LISTEN_RESPONSE_MSG
unsigned short int ConnectStatus;           // Track the connection to access point


tPointerFcn WiFiStateFuncPtr = WiFiState0;  // Start in state 0
WiFi_Properties WiFiParam;                  // Result of NETWORK_STATUS_RESPONSE_MSG
WiFi_Events WiFiEvents;                     // Event flags

char AssignedIP[4];
char ErrorType[2];                          // Error event code
char DataBuffer[128];                       // Buffer for sending data by UDP socket
char CurrentSSID[32];                       // Set by SCAN_RESULT_MSG
char Config[8];                             // Set by SCAN_RESULT_MSG
char ChanNumStr[8];                         // Set by SCAN_RESULT_MSG
unsigned short int ScanIndex;               // Index of scan result
unsigned short int BlinkTimer;              // Another timer handle for blinking label

extern char ACK_Received;                   // Flag for ACK response from MCW1001A


char DIY_MSG[528];  // Do it yourself buffer for making variable data length messages


// Control Messages -------------------------------------------------------------------------------
char RESET_MSG[7] = {0x55,0xAA,0xAA,0x00,0x00,0x00,0x45};
char GPIO_MSG[9] = {0x55,0xAA,0xAC,0x00,0x02,0x00,0x01,0x02,0x45}; // Read index 01 input

/* The following packet prototypes are put in flash to save RAM space                      */
/*                                                   |           Header            |Tra |  */
const code char GET_VERSION_MSG[7]                 = {0x55,0xAA,0x17,0x00,0x00,0x00,0x45};


// Network Configuration Messages -----------------------------------------------------------------
#if USE_DHCP == TRUE
/*                                                   |           Header            | Res|DHCP|       IP address  |                                                           |Tra |  */
const code char SET_IP_ADDRESS_MSG[25]             = {0x55,0xAA,0x29,0x00,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; // IP address  will be assigned by DHCP
#else
/*                                                   |           Header            | Res|Stat|       IP address  |                                                           |Tra |  */
const code char SET_IP_ADDRESS_MSG[25]             = {0x55,0xAA,0x29,0x00,0x12,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; // Static IP address will be set from myIpAdd
#endif
/*                                                   |           Header            |   Network mask    |                                                           |Tra |  */
const code char SET_NETWORK_MASK_MSG[23]           = {0x55,0xAA,0x2A,0x00,0x10,0x00,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; // Network mask will be set from myNetMask
/*                                                   |           Header            |Gateway IP address |                                                           |Tra |  */
const code char SET_GATEWAY_IP_ADDRESS_MSG[23]     = {0x55,0xAA,0x2C,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; // Gateway IP address will be set from gatewayAdd
/*                                                   |           Header            |        MAC address          |Tra |  */
const code char SET_MACADDRESS_MSG[13]             = {0x55,0xAA,0x31,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; // MAC address will be set from myMacAdd
/*                                                   |           Header            |  Time   |Tra |  */
const code char SET_ARP_TIME_MSG[9]                = {0x55,0xAA,0xAD,0x00,0x02,0x00,0x1E,0x00,0x45}; // 30 seconds
/*                                                   |           Header            |Tra |  */
const code char GET_NETWORK_STATUS[7]              = {0x55,0xAA,0x30,0x00,0x00,0x00,0x45};


// WiFi General Configuration Messages ------------------------------------------------------------
/*                                                   |           Header            |Prof|Mode|Tra |  */
const code char SET_CP_NETWORK_MODE_MSG[9]         = {0x55,0xAA,0x37,0x00,0x02,0x00,0x01,0x01,0x45}; // Connection Profile ID: 1, Network Mode: Infrastructure
/*                                                   |           Header            |Prof|Len |       SSID                                                                                        |Tra |  */
const code char SET_CP_SSID_MSG[29]                = {0x55,0xAA,0x39,0x00,0x16,0x00,0x01,0x14,0x4A,0x65,0x61,0x6E,0x27,0x73,0x20,0x57,0x69,0x2D,0x46,0x69,0x20,0x4E,0x65,0x74,0x77,0x6F,0x72,0x6B,0x45}; // Jean's Wi-Fi Network
/*                                                   |           Header            |Code|Res |Tra |  */
const code char SET_REGIONAL_DOMAIN_MSG[9]         = {0x55,0xAA,0x38,0x00,0x02,0x00,0x01,0x00,0x45}; // Country Code: IC - Industry Canada
/*                                                   |           Header            |Chns|Res |          Channel numbers                             |Tra |  */
const code char SET_CHANNEL_LIST_MSG[20]           = {0x55,0xAA,0x3A,0x00,0x0D,0x00,0x0B,0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x45}; // Number of channels in list: 11
/*                                                   |           Header            |Inf |Hock|Tra |  */
const code char SET_LIST_RETRY_COUNT_MSG[9]        = {0x55,0xAA,0x3B,0x00,0x02,0x00,0x05,0x05,0x45}; // Infrastructure count: 5, Ad hock count: 5

// WiFi Security Configuration Messages -----------------------------------------------------------
/*                                                   |           Header            |Prof|Res |Tra |  */
//const code char SET_CP_SECURITY_OPEN_MSG[9]      = {0x55,0xAA,0x41,0x00,0x02,0x00,0x01,0x00,0x45}; // Connection profile ID: 1
/*                                                   |           Header            |Prof|Mode|Ind |Res |         Key 0          |         Key 1          |         Key 2          |         Key 3          |Tra |  */
//const code char SET_CP_SECURITY_WEP40_MSG[31]    = {0x55,0xAA,0x42,0x00,0x18,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; // fill in security keys

/*                                                   |           Header            |Prof|Mode|Ind |Res |                        Key 0                                   |       */
//const code char SET_CP_SECURITY_WEP104_MSG[63]   = {0x55,0xAA,0x43,0x00,0x38,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,       // 13-byte key
/*                                                                                                     |                        Key 1                                   |       */
//                                                                                                      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,       // 13-byte key
/*                                                                                                     |                        Key 2                                   |       */
//                                                                                                      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,       // 13-byte key
/*                                                                                                     |                        Key 3                                   |Tra |  */
//                                                                                                      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45); // 13-byte key


/*                                                   |           Header            |Prof|Type|Res |Len |                  Password                       |Tra |   */
const code char SET_CP_SECURITY_WPA_MSG[21]        = {0x55,0xAA,0x44,0x00,0x0E,0x00,0x01,0x06,0x00,0x0A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45};  // Connection profile: 1, Security type: WPA-2, Password: will be set from myPassword
/*                                                   |           Header            |Prof|Type|Res |Len |                     Key                  */
const code char SET_CP_SECURITY_WPA_MSG_BINARY[43] = {0x55,0xAA,0x44,0x00,0x24,0x00,0x01,0x05,0x00,0x20,0xD6,0x19,0x23,0x50,0xD2,0x26,0xF2,0x06,           // Connection profile: 1, Security type: WPA-2, SSID: Jean's Wi-Fi Network
/*                        |                                        rest of key                                                                    |Tra |   */
                           0x12,0x85,0x8B,0x47,0x17,0x35,0x07,0x06,0x68,0x07,0x6A,0xED,0xBF,0xC6,0xEA,0xD0,0x7E,0x3B,0x47,0x4A,0xE0,0x3F,0xE9,0x09,0x45};  // Binary key obtained from GET_CP_WPAKEY_MSG

//const code char SET_CP_SECURITY_WPA_MSG_BINARY[43] = {0x55,0xAA,0x44,0x00,0x24,0x00,0x01,0x05,0x00,0x20,0xE0,0xBE,0x5C,0x40,0xC4,0x48,0x1C,0x74,
//                           0x0C,0x9B,0x14,0x0A,0xA0,0x61,0x1A,0x7A,0x29,0x01,0xDF,0xE9,0xCF,0x9B,0x7F,0x2F,0xF6,0xD1,0xD2,0x10,0xDB,0x21,0x2E,0xD1,0x45};    // SSID: Aldebaran




/*                                                   |           Header            |Prof|Res |Tra |   */
const code char GET_CP_WPAKEY_MSG[9]               = {0x55,0xAA,0x47,0x00,0x02,0x00,0x01,0x00,0x45};  // Connection profile: 1

// WiFi Scanning Messages -------------------------------------------------------------------------
/*                                                   |           Header            |All |Res |Tra |   */
const code char SCAN_START_MSG[9]                  = {0x55,0xAA,0x50,0x00,0x02,0x00,0xFF,0x00,0x45};  // Scan everything
/*                                                   |           Header            |Ind |Res |Tra |   */
const code char SCAN_GET_RESULTS_MSG[9]            = {0x55,0xAA,0x51,0x00,0x02,0x00,0x00,0x00,0x45};  // Index will be set by application

// WiFi Connection Messages -----------------------------------------------------------------------
/*                                                   |           Header            |Prof|Res |Tra |   */
const code char WIFI_CONNECT_MSG[9]                = {0x55,0xAA,0x5A,0x00,0x02,0x00,0x01,0x00,0x45};  // Connection profile ID: 1
/*                                                   |           Header            |Tra |   */
const code char WIFI_DISCONNECT_MSG[7]             = {0x55,0xAA,0x5B,0x00,0x00,0x00,0x45};



// WiFi Power management Messages -----------------------------------------------------------------
/*                                                   |           Header            |Mode|Res |         |Tra |   */
const code char SET_POWER_MODE_MSG[11]             = {0x55,0xAA,0x66,0x00,0x04,0x00,0x01,0x00,0x0A,0x00,0x45};  // Hibernate

// ICMP (Ping) Messages ---------------------------------------------------------------------------
/*                                                   |           Header            |                  IP address to ping                                           |Tra |  */
const char char PING_SEND_MSG[23]                  = {0x55,0xAA,0x79,0x00,0x10,0x00,0x0A,0x00,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; // Router IP address: 10:0:1:1  (My network IP: 99.242.255.197 = 0x63,0xF2,0xFF,0xC5)

// Socket Messages --------------------------------------------------------------------------------
/*                                                   |           Header            |Type|Res |Tra |   */
const code char SOCKET_CREATE_MSG[9]               = {0x55,0xAA,0x6E,0x00,0x02,0x00,0x00,0x00,0x45};  // Socket type: UDP (alter Type to 0x01 for TCP)
/*                                                   |           Header            |Hndl|Res |Tra |   */
const code char SOCKET_CLOSE_MSG[9]                = {0x55,0xAA,0x6F,0x00,0x02,0x00,0x00,0x00,0x45};  // Alter Hndl to insert socket handle
/*                                                   |           Header            |Loc Port |Hndl|Res |Tra |   NOTE: Microchip engineers put the cart before the horse here (Local Port before the Handle). Totally inconsistent!!!! */
const code char SOCKET_BIND_MSG[11]                = {0x55,0xAA,0x70,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x45};  // Alter Hndl with handle
/*                                                   |           Header            |Hndl|Res |Rem Port |    Remote IP address                                                          |Tra |   */
const code char SOCKET_CONNECT_MSG[27]             = {0x55,0xAA,0x71,0x00,0x14,0x00,0x00,0x00,0x00,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45};  // Byte 6: socket handle, bytes 8,9: remote port, bytes 10..25: remote IP
/*                                                   |           Header            |Hndl|Cnt |Tra |   */
const code char SOCKET_LISTEN_MSG[9]               = {0x55,0xAA,0x72,0x00,0x02,0x00,0x00,0x05,0x45};  // Socket count (backlog) 5
/*                                                   |           Header            |Hndl|Res |Tra |   */
const code char SOCKET_ACCEPT_MSG[9]               = {0x55,0xAA,0x73,0x00,0x02,0x00,0x00,0x00,0x45};
/*                                                   |           Header            |Hndl|Res |Byte Cnt |                   |Tra |   */
const code char SOCKET_SEND_MSG[15]                = {0x55,0xAA,0x74,0x00,0x08,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x45};  // Dummy message. Actual message has to be created dynamically. Byte count little-endian
/*                                                   |           Header            |Hndl|Res |Byte Cnt |Tra |   */
const code char SOCKET_RECEIVE_MSG[11]             = {0x55,0xAA,0x75,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x45};  // Byte count little-endian
/*                                                   |           Header            |Hndl|Res |Rem Port |    Remote IP address                                                          |Byte Cnt |   Send Data       |Tra |  */
const code char SOCKET_SEND_TO_MSG[33]             = {0x55,0xAA,0x76,0x00,0x1A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x45}; // Dummy message. Create dynamically in code
/*                                                   |           Header            |Hndl|Res |Byte Cnt |Tra |   */
const code char SOCKET_RECV_FROM_MSG[11]           = {0x55,0xAA,0x77,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x45};
/*                                                   |           Header            |Cnt1|Cnt2|Buf Size1|Buf Size2|Buf Size3|Buf Size4|Tra |  */
const code char SOCKET_ALLOCATE_MSG[17]            = {0x55,0xAA,0x7A,0x00,0x0A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45}; //
//-------------------------------------------------------------------------------------------------



// Initialization *********************************************************************************
void WiFiPlusInit(void) {
     WiFiPlusHwReset();
     WiFiEvents.NetStatResp = FALSE;
     WiFiTimerHandle = get_timer();    // Reserve a timer for WiFi state machine and never release
}

// Hardware reset MCW1001A chip
void WiFiPlusHwReset(void) {
     GPIO_PORTC_DATA5_bit = 1;         // High
     GPIO_PORTC_DATA5_bit = 0;         // Reset signal active
     Delay_ms(50);                     // Delay
     GPIO_PORTC_DATA5_bit = 1;         // Reset signal inactive
     Delay_ms(100);                    // Delay
}
//-------------------------------------------------------------------------------------------------



// Commands ***************************************************************************************
void WiFiSwReset(void) { // RESET_MSG (170 COMMAND)
     USB_PutQueue("\r\nSending software reset command\r\n");
     WiFiEvents.NetStatResp = FALSE;
     UART0_PutQueue(RESET_MSG,sizeof(RESET_MSG));
}

void WiFiSetIpAddr(void) { // SET_IP_ADDRESS_MSG (41 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set IP address command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_IP_ADDRESS_MSG);i++)
          DIY_MSG[i] = SET_IP_ADDRESS_MSG[i];  // Copy packet
     DIY_MSG[8]  = myIpAdd[0];                 // Insert IP address
     DIY_MSG[9]  = myIpAdd[1];
     DIY_MSG[10] = myIpAdd[2];
     DIY_MSG[11] = myIpAdd[3];
     UART0_PutQueue(DIY_MSG,sizeof(SET_IP_ADDRESS_MSG));
}

void WiFiSetNetworkMask(void) { // SET_NETWORK_MASK_MSG (42 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set network mask command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_NETWORK_MASK_MSG);i++)
          DIY_MSG[i] = SET_NETWORK_MASK_MSG[i];// Copy packet
     DIY_MSG[6] = myNetMask[0];                // Insert network mask
     DIY_MSG[7] = myNetMask[1];
     DIY_MSG[8] = myNetMask[2];
     DIY_MSG[9] = myNetMask[3];
     UART0_PutQueue(DIY_MSG,sizeof(SET_NETWORK_MASK_MSG));
}

void WiFiSetGatewayAddr(void) { // SET_GATEWAY_IP_ADDRESS (44 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set gateway IP address command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_GATEWAY_IP_ADDRESS_MSG);i++)
          DIY_MSG[i] = SET_GATEWAY_IP_ADDRESS_MSG[i];// Copy packet
     DIY_MSG[6] = gatewayAdd[0];               // Insert gateway address
     DIY_MSG[7] = gatewayAdd[1];
     DIY_MSG[8] = gatewayAdd[2];
     DIY_MSG[9] = gatewayAdd[3];
     UART0_PutQueue(DIY_MSG,sizeof(SET_GATEWAY_IP_ADDRESS_MSG));
}

void WiFiSetMacAddr(void) { // SET_MACADDRESS_MSG (49 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set MAC address command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_MACADDRESS_MSG);i++)
          DIY_MSG[i] = SET_MACADDRESS_MSG[i];  // Copy packet
     DIY_MSG[6]  = myMacAdd[0];                // Insert MAC address
     DIY_MSG[7]  = myMacAdd[1];
     DIY_MSG[8]  = myMacAdd[2];
     DIY_MSG[9]  = myMacAdd[3];
     DIY_MSG[10] = myMacAdd[4];
     DIY_MSG[11] = myMacAdd[5];
     UART0_PutQueue(DIY_MSG,sizeof(SET_MACADDRESS_MSG));
}

void WiFiSetArpTime(void) { // SET_ARP_TIME_MSG (173 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set ARP time command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_ARP_TIME_MSG);i++)
          DIY_MSG[i] = SET_ARP_TIME_MSG[i];    // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(SET_ARP_TIME_MSG));
}

void WiFiSetNetworkMode(void) { // SET_CP_NETWORK_MODE_MSG (55 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set network mode command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_CP_NETWORK_MODE_MSG);i++)
          DIY_MSG[i] = SET_CP_NETWORK_MODE_MSG[i]; // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(SET_CP_NETWORK_MODE_MSG));
}

void WiFiSetCpSsid(void) { // SET_CP_SSID_MSG (57 COMMAND)
     unsigned int i;
     unsigned int DataLength;
     unsigned int MsgLength;
     //char *BufNum;

     USB_PutQueue("\r\nSending set CP SSID command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<6;i++)
          DIY_MSG[i] = SET_CP_SSID_MSG[i];     // Insert header
     DIY_MSG[6] = myProfileID;                 // Insert profile ID
     DataLength = strlen(mySSID);              // Get data length
     DIY_MSG[7] = DataLength;                  // Insert SSID length
     for (i=0;i<DataLength;i++)
          DIY_MSG[i+8] = mySSID[i];            // Insert the SSID
     MsgLength = DataLength+2;
     DIY_MSG[4] = MsgLength;                   // Insert message length (little-endian)
     DIY_MSG[5] = (MsgLength >> 8);
     DIY_MSG[MsgLength+6] = 0x45;              // Insert trailer
     UART0_PutQueue(DIY_MSG,MsgLength+7);      // Packet length including header and trailer
/*
     BufNum = GetBuffer();
     sprintf(BufNum,"Header: %02X%02X%02X%02X%02X%02X\r\n",DIY_MSG[0],DIY_MSG[1],DIY_MSG[2],DIY_MSG[3],DIY_MSG[4],DIY_MSG[5]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Profile: %02X, Length: %02X\r\n",DIY_MSG[6],DIY_MSG[7]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"SSID: %02X%02X%02X%02X%02X%02X%02X%02X",DIY_MSG[8],DIY_MSG[9],DIY_MSG[10],DIY_MSG[11],DIY_MSG[12],DIY_MSG[13],DIY_MSG[14],DIY_MSG[15]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"SSID: %02X%02X%02X%02X%02X%02X%02X%02X",DIY_MSG[16],DIY_MSG[17],DIY_MSG[18],DIY_MSG[19],DIY_MSG[20],DIY_MSG[21],DIY_MSG[22],DIY_MSG[23]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"SSID: %02X%02X%02X%02X\r\n",DIY_MSG[24],DIY_MSG[25],DIY_MSG[26],DIY_MSG[27]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Trailer: %02X\r\n",DIY_MSG[28]);
     USB_PutQueue(BufNum);
*/
}

void WiFiSetRegionalDomain(void) {  // SET_REGIONAL_DOMAIN_MSG (56 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set regional domain command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_REGIONAL_DOMAIN_MSG);i++)
          DIY_MSG[i] = SET_REGIONAL_DOMAIN_MSG[i]; // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(SET_REGIONAL_DOMAIN_MSG));
}

void WiFiSetChannelList(void) { // SET_CHANNEL_LIST_MSG (58 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set channel list command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_CHANNEL_LIST_MSG);i++)
          DIY_MSG[i] = SET_CHANNEL_LIST_MSG[i]; // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(SET_CHANNEL_LIST_MSG));
}

void WiFiSetListRetryCount(void) { // SET_LIST_RETRY_COUNT_MSG (59 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending set list retry count command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_LIST_RETRY_COUNT_MSG);i++)
          DIY_MSG[i] = SET_LIST_RETRY_COUNT_MSG[i]; // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(SET_LIST_RETRY_COUNT_MSG));
}

void WiFiSetSecurityWpa(void) { // SET_CP_SECURITY_WPA_MSG (68 COMMAND)
     unsigned int i;
     unsigned int DataLength;
     unsigned int MsgLength;

     USB_PutQueue("\r\nSending set security WPA password command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<6;i++)
          DIY_MSG[i] = SET_CP_SECURITY_WPA_MSG[i]; // Insert header
     DIY_MSG[6] = myProfileID;                 // Insert profile ID
     DIY_MSG[7] = 0x06;                        // Type
     DataLength = strlen(myPassword);          // Get password length
     DIY_MSG[9] = DataLength;                  // Insert password length
     for (i=0;i<DataLength;i++)
          DIY_MSG[i+10] = myPassword[i];       // Insert the password
     MsgLength = DataLength+4;
     DIY_MSG[4] = MsgLength;                   // Insert message length (little-endian)
     DIY_MSG[5] = (MsgLength >> 8);
     DIY_MSG[MsgLength+6] = 0x45;              // Insert trailer
     UART0_PutQueue(DIY_MSG,MsgLength+7);      // Packet length including header and trailer
}

void WiFiSetSecurityWpaBinary(void) { // SET_CP_SECURITY_WPA_MSG (68 COMMAND with 32-bit binary key)
     unsigned int i;

     USB_PutQueue("\r\nSending set security WPA binary command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SET_CP_SECURITY_WPA_MSG_BINARY);i++)
          DIY_MSG[i] = SET_CP_SECURITY_WPA_MSG_BINARY[i]; // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(SET_CP_SECURITY_WPA_MSG_BINARY));
}

void WiFiScanStart(void) { // SCAN_START_MSG (80 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending scan start command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SCAN_START_MSG);i++)
          DIY_MSG[i] = SCAN_START_MSG[i];      // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(SCAN_START_MSG));
}

void WiFiConnect(void) { // Wi-Fi_CONNECT_MSG (90 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending connect command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(WIFI_CONNECT_MSG);i++)
          DIY_MSG[i] = WIFI_CONNECT_MSG[i];    // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(WIFI_CONNECT_MSG));
}

void WiFiDisconnect(void) { // Wi-Fi_DISCONNECT_MSG (91 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending disconnect command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(WIFI_DISCONNECT_MSG);i++)
          DIY_MSG[i] = WIFI_DISCONNECT_MSG[i]; // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(WIFI_DISCONNECT_MSG));
}

void WiFiPingSend(void) { // PING_SEND_MSG (121 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending ping command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(PING_SEND_MSG);i++)
          DIY_MSG[i] = PING_SEND_MSG[i];       // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(PING_SEND_MSG));
}
//-------------------------------------------------------------------------------------------------

void WiFiSocketCreate(char SocType) {
     unsigned int i;

     if (SocType == UDP)
         USB_PutQueue("\r\nCreating UDP socket\r\n");
     if (SocType == TCP)
         USB_PutQueue("\r\nCreating TCP socket\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_CREATE_MSG);i++)
          DIY_MSG[i] = SOCKET_CREATE_MSG[i];   // Copy packet
     DIY_MSG[6] = SocType;                     // Insert socket type
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_CREATE_MSG));
}

void WiFiSocketClose(char Handle) {
     unsigned int i;

     USB_PutQueue("\r\nSending socket close command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_CLOSE_MSG);i++)
          DIY_MSG[i] = SOCKET_CLOSE_MSG[i];    // Copy packet
     DIY_MSG[6] = Handle;                      // Insert handle
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_CLOSE_MSG));
}

void WiFiSocketBind(char Handle, unsigned int LocalPort) {
     unsigned int i;
     char *BufNum;

     USB_PutQueue("\r\nSending socket bind command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_BIND_MSG);i++)
          DIY_MSG[i] = SOCKET_BIND_MSG[i];     // Copy packet
     DIY_MSG[8] = Handle;                      // Insert handle
     DIY_MSG[6] = LocalPort;                   // Insert local port (little-endian)
     DIY_MSG[7] = (LocalPort >> 8);
#if USB_Debug == TRUE
     BufNum = GetBuffer();
     sprintf(BufNum,"Local port: %02X%02X\r\n",SOCKET_BIND_MSG[6],SOCKET_BIND_MSG[7]);
     USB_PutQueue(BufNum);
#endif
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_BIND_MSG));
}

void WiFiSocketConnect(char Handle) {
     unsigned int i;

     USB_PutQueue("\r\nSending socket connect command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_CONNECT_MSG);i++)
          DIY_MSG[i] = SOCKET_CONNECT_MSG[i];  // Copy packet
     DIY_MSG[6] = Handle;                      // Insert handle
     DIY_MSG[8] = UDP_RemPort;                 // Insert remote port (little-endian)
     DIY_MSG[9] = (UDP_RemPort >> 8);
     DIY_MSG[10] = UDP_RemoteIP[0];            // Insert remote IP address
     DIY_MSG[11] = UDP_RemoteIP[1];
     DIY_MSG[12] = UDP_RemoteIP[2];
     DIY_MSG[13] = UDP_RemoteIP[3];
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_CONNECT_MSG));
}

void WiFiSocketListen(char Handle) {
     unsigned int i;

     USB_PutQueue("\r\nSending socket Listen command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_LISTEN_MSG);i++)
          DIY_MSG[i] = SOCKET_LISTEN_MSG[i];   // Copy packet
     DIY_MSG[6] = Handle;                      // Insert handle
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_LISTEN_MSG));
}

void WiFiSocketAccept(char Handle) {
     unsigned int i;

     USB_PutQueue("\r\nSending socket accept command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_ACCEPT_MSG);i++)
          DIY_MSG[i] = SOCKET_ACCEPT_MSG[i];   // Copy packet
     DIY_MSG[6] = Handle;                      // Insert handle
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_ACCEPT_MSG));
}

void WiFiSocketSend(char Handle, char *Data) {
     unsigned int i;
     unsigned int DataLength;
     unsigned int MsgLength;
     //char *BufNum;

     USB_PutQueue("\r\nSending socket send command\r\n");
     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<6;i++)
          DIY_MSG[i] = SOCKET_SEND_MSG[i];     // Insert header
     DIY_MSG[6] = Handle;                      // Insert handle
     DataLength = strlen(Data);                // Get data length
     DIY_MSG[8] = DataLength;                  // Insert data length (little-endian)
     DIY_MSG[9] = (DataLength >> 8);
     for (i=0;i<DataLength;i++)
          DIY_MSG[i+10] = Data[i];             // Insert the data
     MsgLength = DataLength+4;
     DIY_MSG[4] = MsgLength;                   // Insert message length (little-endian)
     DIY_MSG[5] = (MsgLength >> 8);
     DIY_MSG[MsgLength+6] = 0x45;              // Insert trailer
     UART0_PutQueue(DIY_MSG,MsgLength+7);      // Packet length including header and trailer
/*
     BufNum = GetBuffer();
     sprintf(BufNum,"Data length: %u, Message length: %u\r\n",DataLength,MsgLength);
     USB_PutQueue(BufNum);
*/
/*
     BufNum = GetBuffer();
     sprintf(BufNum,"Header: %02X%02X%02X%02X%02X%02X",DIY_MSG[0],DIY_MSG[1],DIY_MSG[2],DIY_MSG[3],DIY_MSG[4],DIY_MSG[5]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Handle: %02X, Reserved: %02X, Byte count: %02X%02X",DIY_MSG[6],DIY_MSG[7],DIY_MSG[8],DIY_MSG[9]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Data: %02X%02X%02X%02X",DIY_MSG[10],DIY_MSG[11],DIY_MSG[12],DIY_MSG[13]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Trailer: %02X\r\n",DIY_MSG[14]);
     USB_PutQueue(BufNum);
*/
}

void WiFiSocketReceive(char Handle, unsigned int DataLength) {
     unsigned int i;

     USB_PutQueue("\r\nSending socket receive command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_RECEIVE_MSG);i++)
          DIY_MSG[i] = SOCKET_RECEIVE_MSG[i];  // Copy packet
     DIY_MSG[6] = Handle;                      // Insert handle
     DIY_MSG[8] = DataLength;                  // Insert data length (little-endian)
     DIY_MSG[9] = (DataLength >> 8);
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_RECEIVE_MSG));
}

void WiFiSocketSendTo(char Handle, char *Data) {
     unsigned int i;
     unsigned int DataLength;
     unsigned int MsgLength;
     char *BufNum;

     USB_PutQueue("\r\nSending socket send to command\r\n");
     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<6;i++)
          DIY_MSG[i] = SOCKET_SEND_TO_MSG[i];  // Insert header
     DIY_MSG[6] = Handle;                      // Insert handle
     DIY_MSG[8] = UDP_RemPort;                 // Insert remote port (little-endian)
     DIY_MSG[9] = (UDP_RemPort >> 8);

     DIY_MSG[10] = UDP_RemoteIP[0];            // Insert remote IP address
     DIY_MSG[11] = UDP_RemoteIP[1];
     DIY_MSG[12] = UDP_RemoteIP[2];
     DIY_MSG[13] = UDP_RemoteIP[3];

     DataLength = strlen(Data);                // Get data length
     DIY_MSG[26] = DataLength;                 // Insert data length (little-endian)
     DIY_MSG[27] = (DataLength >> 8);
     for (i=0;i<DataLength;i++)
          DIY_MSG[i+28] = Data[i];             // Insert the data
     MsgLength = DataLength+22;
     DIY_MSG[4] = MsgLength;                   // Insert message length (little-endian)
     DIY_MSG[5] = (MsgLength >> 8);
     DIY_MSG[MsgLength+6] = 0x45;              // Insert trailer
     UART0_PutQueue(DIY_MSG,MsgLength+7);      // Packet length including header and trailer

/*                              |           Header            |Hndl|Res |Rem Port |    Remote IP address                                                          |Byte Cnt |   Send Data                      |Tra |  */
//char SOCKET_SEND_TO_MSG[33] = {0x55,0xAA,0x76,0x00,0x1A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x45};
//                               0    1    2    3    4    5    6    7    8    9    10   11   12   13   14   15   16   17   18   19   20   21   22   23   24   25   26   27   28   29   30   31   32   33   34   35

     BufNum = GetBuffer();
     sprintf(BufNum,"Header: %02X%02X%02X%02X%02X%02X",DIY_MSG[0],DIY_MSG[1],DIY_MSG[2],DIY_MSG[3],DIY_MSG[4],DIY_MSG[5]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Handle: %02X, Reserved: %02X, Remote port: %02X%02X",DIY_MSG[6],DIY_MSG[7],DIY_MSG[8],DIY_MSG[9]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Remote IP address: %02X%02X%02X%02X",DIY_MSG[10],DIY_MSG[11],DIY_MSG[12],DIY_MSG[13]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Date byte count: %02X%02X",DIY_MSG[26],DIY_MSG[27]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Data: %02X%02X%02X%02X%02X%02X%02X",DIY_MSG[28],DIY_MSG[29],DIY_MSG[30],DIY_MSG[31],DIY_MSG[32],DIY_MSG[33],DIY_MSG[34]);
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Trailer: %02X\r\n",DIY_MSG[35]);
     USB_PutQueue(BufNum);


}

void WiFiSocketReceiveFrom(char Handle, unsigned int DataLength) {
     unsigned int i;

     USB_PutQueue("\r\nSending socket receive from command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_RECV_FROM_MSG);i++)
          DIY_MSG[i] = SOCKET_RECV_FROM_MSG[i];// Copy packet
     DIY_MSG[6] = Handle;                      // Insert handle
     DIY_MSG[8] = DataLength;                  // Insert data length (little-endian)
     DIY_MSG[9] = (DataLength >> 8);
     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_RECV_FROM_MSG));
}

void WiFiSocketAllocate(void) {
     unsigned int i;

     USB_PutQueue("\r\nSending socket allocate command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SOCKET_ALLOCATE_MSG);i++)
          DIY_MSG[i] = SOCKET_ALLOCATE_MSG[i]; // Copy packet
     DIY_MSG[6] = TCP_ServerSocketCount;       // Insert TCP Server Socket Count
     DIY_MSG[7] = TCP_ClientSocketCount;       // Insert TCP Client Socket Count

     DIY_MSG[8] = TCP_ServerRcvBufSize;        // Insert TCP Server Socket Buffer Size
     DIY_MSG[9] = (TCP_ServerRcvBufSize >> 8);
     DIY_MSG[10] = TCP_ServerXmtBufSize;       // Insert TCP Server Socket Transmit Buffer Size
     DIY_MSG[11] = (TCP_ServerXmtBufSize >> 8);

     DIY_MSG[12] = TCP_ClientRcvBufSize;       // Insert TCP Client Socket Buffer Size
     DIY_MSG[13] = (TCP_ClientRcvBufSize >> 8);
     DIY_MSG[14] = TCP_ClientXmtBufSize;       // Insert TCP Client Socket Transmit Buffer Size
     DIY_MSG[15] = (TCP_ClientXmtBufSize >> 8);

     UART0_PutQueue(DIY_MSG,sizeof(SOCKET_ALLOCATE_MSG));
}

//-------------------------------------------------------------------------------------------------




// Requests ***************************************************************************************
void WiFiGetVersion(void) { // GET_VERSION_MSG (23 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending get version command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(GET_VERSION_MSG);i++)
          DIY_MSG[i] = GET_VERSION_MSG[i];     // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(GET_VERSION_MSG));
}

void WiFiGetNetworkStatus(void) { // GET_NETWORK_STATUS_MSG (48 COMMAND)
     unsigned int i;
     //char *BufNum;

     USB_PutQueue("\r\nSending get network status command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(GET_NETWORK_STATUS);i++)
          DIY_MSG[i] = GET_NETWORK_STATUS[i];     // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(GET_NETWORK_STATUS));

/*                                        |           Header            |Tra |  */
//const code char GET_NETWORK_STATUS[7] = {0x55,0xAA,0x30,0x00,0x00,0x00,0x45};
/*
     BufNum = GetBuffer();
     sprintf(BufNum,"Message size: %02X",sizeof(GET_NETWORK_STATUS));
     USB_PutQueue(BufNum);

     BufNum = GetBuffer();
     sprintf(BufNum,"Packet: %02X%02X%02X%02X%02X%02X%02X",DIY_MSG[0],DIY_MSG[1],DIY_MSG[2],DIY_MSG[3],DIY_MSG[4],DIY_MSG[5],DIY_MSG[6]);
     USB_PutQueue(BufNum);
*/
}

void WiFiGetWpaKey(void) { // GET_CP_WPAKEY_MSG (71 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending get WPA key command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(GET_CP_WPAKEY_MSG);i++)
          DIY_MSG[i] = GET_CP_WPAKEY_MSG[i];     // Copy packet
     UART0_PutQueue(DIY_MSG,sizeof(GET_CP_WPAKEY_MSG));
}

void WiFiScanGetResult(unsigned short int Index) { // SCAN_GET_RESULT_MSG (81 COMMAND)
     unsigned int i;

     USB_PutQueue("\r\nSending get scan result command\r\n");

     memset(DIY_MSG,0,528);                    // Clear out buffer
     for (i=0;i<sizeof(SCAN_GET_RESULTS_MSG);i++)
          DIY_MSG[i] = SCAN_GET_RESULTS_MSG[i];// Copy packet
     DIY_MSG[6] = Index;                       // Insert index
     UART0_PutQueue(DIY_MSG,sizeof(SCAN_GET_RESULTS_MSG));
}
//-------------------------------------------------------------------------------------------------





//*************************************************************************************************
//*************************************************************************************************
// Call this function in main loop
void WiFiApp(void) {
     WiFiStateFuncPtr();  // Go to WiFi application state
}

// Hold here checking the error response if any
void WiFiState0(void) {   // Hold here while checking for errors
     if (WiFiEvents.Error == TRUE) WiFiStateFuncPtr = WiFiState666;  // Go to error handler
}
//-------------------------------------------------------------------------------------------------


/************ Sequences invoked by USB_Driver.c for testing the protocol *************************/
// Sequence to fetch network status ***************************************************************
// Issue get network status command
void WiFiState1(void) {
     WiFiEvents.NetStatResp = FALSE;           // Clear network status response flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiGetNetworkStatus();                   // Issue get network status command
     WiFiStateFuncPtr = WiFiState2;
}

// Wait for network status to be received
void WiFiState2(void) {
     if (WiFiEvents.NetStatResp == TRUE) {     // Waiting for network status response
         USB_PutQueue("Network status available\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(2);
}
//-------------------------------------------------------------------------------------------------


// Sequence to scan for nearby access points ******************************************************
// Proceed with scan only if not connected to access point
void WiFiState4(void) {
     if ((WiFiEvents.NetStatResp == TRUE) && ((WiFiParam.NetworkStatus == 0) || (WiFiParam.NetworkStatus == 2))) {
          USB_PutQueue("Starting access point scan\r\n");
          WiFiStateFuncPtr = WiFiState5;       // WiFi can perform scan
     }else{
          USB_PutQueue("WiFi not ready\r\n");
          WiFiStateFuncPtr = WiFiState0;       // Return to holding state
     }
}

// Issue scan start command
void WiFiState5(void) {
     ACK_Received = FALSE;                     // Clear ACK flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiScanStart();                          // Request access point scan
     WiFiStateFuncPtr = WiFiState6;            // Go wait for ACK
}

// Wait for initial ACK to be received
void WiFiState6(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         set_timer(WiFiTimerHandle,10);        // Reset timeout period
         WiFiEvents.ScnRsltRdy = FALSE;        // Clear scan result ready flag in preparation for new data
         ClearAllLabels();
         WiFiStateFuncPtr = WiFiState7;        // Go wait for scan result ready message
     }
     TimeoutCheck(6);
}

// Wait for scan result ready message to be received
void WiFiState7(void) {
     if (WiFiEvents.ScnRsltRdy == TRUE) {      // Waiting for scan result ready flag
         ScanIndex = 0;                        // Start off with index of 0
         WiFiStateFuncPtr = WiFiState8;        // Go wait for scan result message
     }
     TimeoutCheck(7);
}

// Issue a retrieve scan result command
void WiFiState8(void) {
     if (NumScanRslts == 0) {                  // If there are no scan results
         USB_PutQueue("\r\nNo more scan results\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }else{
         set_timer(WiFiTimerHandle,10);        // Reset timeout period
         WiFiEvents.ScnRsltMsg = FALSE;        // Clear scan result message flag
         WiFiScanGetResult(ScanIndex);         // Get scan result command
         WiFiStateFuncPtr = WiFiState9;        // Go wait for message
     }
}

// Wait for scan result message to be received
void WiFiState9(void) {
     if (WiFiEvents.ScnRsltMsg == TRUE) {      // Waiting for scan result message

         switch (ScanIndex) {
             case 0:
                 UpdateLabel4(&CurrentSSID);
                 UpdateLabel5(&Config);
                 UpdateLabel6(&ChanNumStr);
                 break;
             case 1:
                 UpdateLabel7(&CurrentSSID);
                 UpdateLabel8(&Config);
                 UpdateLabel9(&ChanNumStr);
                 break;
             case 2:
                 UpdateLabel10(&CurrentSSID);
                 UpdateLabel11(&Config);
                 UpdateLabel12(&ChanNumStr);
                 break;
             case 3:
                 UpdateLabel13(&CurrentSSID);
                 UpdateLabel14(&Config);
                 UpdateLabel15(&ChanNumStr);
                 break;
             case 4:
                 UpdateLabel16(&CurrentSSID);
                 UpdateLabel15(&Config);
                 UpdateLabel18(&ChanNumStr);
                 break;

             default:
                 break;
         }

         ScanIndex += 1;                       // Increment index
         NumScanRslts--;                       // Decrement number of scan results
         WiFiStateFuncPtr = WiFiState8;        // Go back and check for more messages
     }
     TimeoutCheck(9);
}
//-------------------------------------------------------------------------------------------------


// Sequence to set MAC addresses, IP address, Network mask, Gateway IP address, Network mode,
// SSID, Regional domain, Channel list, List retry count, Security WPA-2 **************************
// First set the MAC address
void WiFiState15(void) {
     ACK_Received = FALSE;                     // Clear ACK flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiSetMacAddr();                         // Set MAC address command
     WiFiStateFuncPtr = WiFiState16;           // Go wait for ACK
}

// Wait for ACK to be received then send set IP address command
void WiFiState16(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetIpAddr();                      // Set IP address command
         WiFiStateFuncPtr = WiFiState17;       // Go wait for ACK
     }
     TimeoutCheck(16);
}

// Wait for ACK to be received then send set network mask command
void WiFiState17(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetNetworkMask();                 // Set network mask command
         WiFiStateFuncPtr = WiFiState18;       // Go wait for ACK
     }
     TimeoutCheck(17);
}

// Wait for ACK to be received then send set gateway IP address command
void WiFiState18(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetGatewayAddr();                 // Set gateway address command
         WiFiStateFuncPtr = WiFiState19;       // Go wait for ACK
     }
     TimeoutCheck(18);
}

// Wait for ACK to be received then send set network mode command
void WiFiState19(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetNetworkMode();                 // Set network mode command
         WiFiStateFuncPtr = WiFiState20;       // Go wait for ACK
     }
     TimeoutCheck(19);
}

// Wait for ACK to be received then send set SSID command
void WiFiState20(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);        // Set timeout to 4 seconds
         WiFiSetCpSsid();                      // Set SSID command
         WiFiStateFuncPtr = WiFiState21;       // Go wait for ACK
     }
     TimeoutCheck(20);
}

// Wait for ACK to be received then send set regional domain command
void WiFiState21(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetRegionalDomain();              // Set regional domain command
         WiFiStateFuncPtr = WiFiState22;       // Go wait for ACK
     }
     TimeoutCheck(21);
}

// Wait for ACK to be received then send set channel list command
void WiFiState22(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetChannelList();                 // Set channel list command
         WiFiStateFuncPtr = WiFiState23;       // Go wait for ACK
     }
     TimeoutCheck(22);
}

// Wait for ACK to be received then send set channel list retry count command
void WiFiState23(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetListRetryCount();              // Set channel list command
         WiFiStateFuncPtr = WiFiState24;       // Go wait for ACK
     }
     TimeoutCheck(23);
}

#if USE_PASSWORD
// Wait for ACK to be received then send set security command (using password)
void WiFiState24(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetSecurityWpa();                 // Set security command with password
         WiFiStateFuncPtr = WiFiState25;       // Go wait for ACK
     }
     TimeoutCheck(24);
}
#else
// Wait for ACK to be received then send set security command (using 32 bit binary key)
void WiFiState24(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetSecurityWpaBinary();           // Set security command with binary key
         WiFiStateFuncPtr = WiFiState25;       // Go wait for ACK
     }
     TimeoutCheck(24);
}
#endif

// Wait for ACK to be received then finish
void WiFiState25(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(25);
}
//-------------------------------------------------------------------------------------------------



// Sequence to connect to access point ************************************************************
void WiFiState30(void) {
     ACK_Received = FALSE;                     // Clear ACK flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiConnect();                            // Issue connect command
     WiFiStateFuncPtr = WiFiState31;           // Go wait for ACK
}

void WiFiState31(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(31);
}
//-------------------------------------------------------------------------------------------------



// Sequence to disconnect from access point *******************************************************
void WiFiState35(void) {
     ACK_Received = FALSE;                     // Clear ACK flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiDisconnect();                         // Issue disconnect command
     WiFiStateFuncPtr = WiFiState36;           // Go wait for ACK
}

void WiFiState36(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(36);
}
//-------------------------------------------------------------------------------------------------



// Sequence to create UDP socket ******************************************************************
// Issue socket create command
void WiFiState40(void) {
     WiFiEvents.SockCreaResp = FALSE;          // Clear socket create response flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiSocketCreate(UDP);                    // Issue socket create command
     WiFiStateFuncPtr = WiFiState41;
}

// Wait for socket create response to be received
void WiFiState41(void) {
     if (WiFiEvents.SockCreaResp == TRUE) {    // Wait for socket create response flag
         if (SocketHandle < 254) HandleValid = TRUE;
         else HandleValid = FALSE;
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(41);
}
//-------------------------------------------------------------------------------------------------



// Sequence to close a socket *********************************************************************
void WiFiState45(void) {
     if (HandleValid == TRUE) {
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketClose(SocketHandle);        // Issue socket close command
         WiFiStateFuncPtr = WiFiState46;       // Go wait for ACK
     }else{
         USB_PutQueue("No valid socket open\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
}

void WiFiState46(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(46);
}
//-------------------------------------------------------------------------------------------------



// Sequence to bind a socket **********************************************************************
void WiFiState50(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockBindResp = FALSE;      // Clear socket bind response flag
         BindResult = 255;                     // Invalidate bind result
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketBind(SocketHandle,UDP_LocPort);  // Issue socket bind command
         WiFiStateFuncPtr = WiFiState51;       // Go wait for socket bind response
     }else{
         USB_PutQueue("No valid socket open\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
}

void WiFiState51(void) {
     if (WiFiEvents.SockBindResp == TRUE) {
         if (BindResult == 0)
             USB_PutQueue("Bind successful\r\n");
         else
             USB_PutQueue("Bind failed\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(51);
}
//-------------------------------------------------------------------------------------------------


// Sequence to connect a remote UDP port **********************************************************
void WiFiState55(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockConnResp = FALSE;      // Clear socket connect response flag
         ConnectResult = 255;                  // Invalidate connect result
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketConnect(SocketHandle);      // Issue socket connect command
         WiFiStateFuncPtr = WiFiState56;       // Go wait for socket connect response
     }else{
         USB_PutQueue("No valid socket open\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
}

void WiFiState56(void) {
     if (WiFiEvents.SockConnResp == TRUE) {
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(56);
}
//-------------------------------------------------------------------------------------------------



// Sequence to put a socket into listening mode ***************************************************
void WiFiState60(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockListenResp = FALSE;    // Clear socket listen response flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketListen(SocketHandle);       // Issue socket listen command
         WiFiStateFuncPtr = WiFiState61;       // Go wait for socket connect response
     }else{
         USB_PutQueue("No valid socket open\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
}

void WiFiState61(void) {
     if (WiFiEvents.SockListenResp == TRUE) {
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(61);
}
//-------------------------------------------------------------------------------------------------


// Socket accept command **************************************************************************
void WiFiState62(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockAccptResp = FALSE;     // Clear socket accept response flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketAccept(SocketHandle);       // Issue socket accept command
         WiFiStateFuncPtr = WiFiState62;       // Go wait for socket connect response
     }else{
         USB_PutQueue("No valid socket open\r\n");
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
}

void WiFiState63(void) {
     if (WiFiEvents.SockListenResp == TRUE) {
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(63);
}
//-------------------------------------------------------------------------------------------------


// Socket send command ****************************************************************************
void WiFiState65(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockSendResp = FALSE;      // Clear socket send response flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketSend(SocketHandle,&DataBuffer);
         WiFiStateFuncPtr = WiFiState66;       // Go wait for socket send response
     }
}

void WiFiState66(void) {
     if (WiFiEvents.SockSendResp == TRUE) {
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(66);
}
//-------------------------------------------------------------------------------------------------


// Socket receive command *************************************************************************
void WiFiState67(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockRcvResp = FALSE;       // Clear socket receive response flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketReceive(SocketHandle,128);  // Data length
         WiFiStateFuncPtr = WiFiState68;       // Go wait for socket send response
     }
}

void WiFiState68(void) {
     if (WiFiEvents.SockRcvResp == TRUE) {
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(68);
}
//-------------------------------------------------------------------------------------------------


// Socket send to command *************************************************************************
void WiFiState70(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockSendToResp = FALSE;    // Clear socket send to response flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSocketSendTo(SocketHandle,&DataBuffer);
         WiFiStateFuncPtr = WiFiState71;       // Go wait for socket send to response
     }
}

void WiFiState71(void) {
     if (WiFiEvents.SockSendToResp == TRUE) {
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(71);
}
//-------------------------------------------------------------------------------------------------


// Socket receive from command ********************************************************************
void WiFiState74(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockRcvFromResp = FALSE;   // Clear socket receive from response flag
         set_timer(WiFiTimerHandle,120);       // Set timeout to 120 seconds
         WiFiSocketReceiveFrom(SocketHandle,128); // Data length
         WiFiStateFuncPtr = WiFiState75;       // Go wait for socket receive from response
     }
}

void WiFiState75(void) {
     if (WiFiEvents.SockRcvFromResp == TRUE) {
         WiFiStateFuncPtr = WiFiState74;        // Return to holding state
     }
     TimeoutCheck(75);
}
//-------------------------------------------------------------------------------------------------


// Sequence that implements a UDP server **********************************************************
void WiFiState80(void) {
     if (HandleValid == TRUE) {
         WiFiEvents.SockRcvFromResp = FALSE;   // Clear socket receive from response flag
         WiFiSocketReceiveFrom(SocketHandle,128); // Data length
         WiFiStateFuncPtr = WiFiState81;       // Go wait for socket receive from response
     }
}

void WiFiState81(void) {
     if (WiFiEvents.SockRcvFromResp == TRUE) {
         switch (UDP_Buf[0]) {
             case 'A':
                 strcpy(DataBuffer,"Command A received\r\n");
                 break;

             case 'B':
                 strcpy(DataBuffer,"Command B received\r\n");
                 break;

             default:
                 strcpy(DataBuffer,"Unimplemented command\r\n");
                 break;
         }

         WiFiEvents.SockSendToResp = FALSE;    // Clear socket send to response flag
         WiFiSocketSendTo(SocketHandle,&DataBuffer);
         WiFiStateFuncPtr = WiFiState82;       // Go wait for socket send to response
     }
}

void WiFiState82(void) {
     if (WiFiEvents.SockSendToResp == TRUE) {
         WiFiEvents.SockRcvFromResp = FALSE;   // Clear socket receive from response flag
         WiFiSocketReceiveFrom(SocketHandle,128);
         WiFiStateFuncPtr = WiFiState81;       // Go wait for socket receive from response
     }
}
//-------------------------------------------------------------------------------------------------


/************************ Sequences invoked by TFT LCD touch panel *******************************/
// Combined sequence to get network status and scan for access points *****************************
// This sequence is called when the Scan button is pressed on the LCD.
// Issue get network status command
void WiFiState90(void) {
     WiFiEvents.NetStatResp = FALSE;           // Clear network status response flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiGetNetworkStatus();                   // Issue get network status command
     WiFiStateFuncPtr = WiFiState91;           // Go to next state
}

// Wait for network status to be received
void WiFiState91(void) {
     if (WiFiEvents.NetStatResp == TRUE) {     // Waiting for network status response
         USB_PutQueue("Network status available\r\n");
         // Check to see if disconnected from access point
         if ((WiFiParam.NetworkStatus == 0) || (WiFiParam.NetworkStatus == 2)) {
              WiFiStateFuncPtr = WiFiState93;  // Bypass next state
         }else{
              USB_PutQueue("Disconnecting WiFi\r\n");
              ACK_Received = FALSE;            // Clear ACK flag
              set_timer(WiFiTimerHandle,4);    // Set timeout to 4 seconds
              WiFiDisconnect();                // Issue disconnect command
              WiFiStateFuncPtr = WiFiState92;  // Go to next state
         }
     }
     TimeoutCheck(91);
}

// Wait for disconnect ACK
void WiFiState92(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         WiFiStateFuncPtr = WiFiState93;       // Go to next state
     }
     TimeoutCheck(92);
}

// Issue scan start command
void WiFiState93(void) {
     USB_PutQueue("\r\nStarting access point scan\r\n");
     ACK_Received = FALSE;                     // Clear ACK flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiScanStart();                          // Request access point scan
     WiFiStateFuncPtr = WiFiState94;           // Go wait for ACK
}

// Wait for initial ACK to be received
void WiFiState94(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         set_timer(WiFiTimerHandle,10);        // Reset timeout period
         WiFiEvents.ScnRsltRdy = FALSE;        // Clear scan result ready flag in preparation for new data
         ClearAllLabels();
         WiFiStateFuncPtr = WiFiState95;       // Go wait for scan result ready message
     }
     TimeoutCheck(94);
}

// Wait for scan result ready message to be received
void WiFiState95(void) {
     if (WiFiEvents.ScnRsltRdy == TRUE) {      // Waiting for scan result ready flag
         ScanIndex = 0;                        // Start off with index of 0
         WiFiStateFuncPtr = WiFiState96;       // Go wait for scan result message
     }
     TimeoutCheck(95);
}

// Issue a retrieve scan result command
void WiFiState96(void) {
     if (NumScanRslts == 0) {                  // If there are no scan results
         USB_PutQueue("\r\nNo more scan results\r\n");
         EnableButton2();
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }else{
         set_timer(WiFiTimerHandle,10);        // Reset timeout period
         WiFiEvents.ScnRsltMsg = FALSE;        // Clear scan result message flag
         WiFiScanGetResult(ScanIndex);         // Get scan result command
         WiFiStateFuncPtr = WiFiState97;       // Go wait for message
     }
}

// Wait for scan result message to be received
void WiFiState97(void) {
     if (WiFiEvents.ScnRsltMsg == TRUE) {      // Waiting for scan result message

         switch (ScanIndex) {
             case 0:
                 UpdateLabel4(&CurrentSSID);
                 UpdateLabel5(&Config);
                 UpdateLabel6(&ChanNumStr);
                 break;
             case 1:
                 UpdateLabel7(&CurrentSSID);
                 UpdateLabel8(&Config);
                 UpdateLabel9(&ChanNumStr);
                 break;
             case 2:
                 UpdateLabel10(&CurrentSSID);
                 UpdateLabel11(&Config);
                 UpdateLabel12(&ChanNumStr);
                 break;
             case 3:
                 UpdateLabel13(&CurrentSSID);
                 UpdateLabel14(&Config);
                 UpdateLabel15(&ChanNumStr);
                 break;
             case 4:
                 UpdateLabel16(&CurrentSSID);
                 UpdateLabel17(&Config);
                 UpdateLabel18(&ChanNumStr);
                 break;

             default:                          // Display on LCD maximum of 5 access points
                 break;
         }
         ScanIndex += 1;                       // Increment index
         NumScanRslts--;                       // Decrement number of scan results
         WiFiStateFuncPtr = WiFiState96;       // Go back and check for more messages
     }
     TimeoutCheck(97);
}
//-------------------------------------------------------------------------------------------------



// Combined sequence to connect WiFi Plus to an access point **************************************
// This sequence is called when the Connect button is pressed on the LCD.
// First set the MAC address
void WiFiState100(void) {
     ACK_Received = FALSE;                     // Clear ACK flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiSetMacAddr();                         // Set MAC address command
     WiFiStateFuncPtr = WiFiState101;          // Go wait for ACK
}

// Wait for ACK to be received then send set IP address command
void WiFiState101(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetIpAddr();                      // Set IP address command
         WiFiStateFuncPtr = WiFiState102;      // Go wait for ACK
     }
     TimeoutCheck(101);
}

// Wait for ACK to be received then send set network mask command
void WiFiState102(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetNetworkMask();                 // Set network mask command
         WiFiStateFuncPtr = WiFiState103;      // Go wait for ACK
     }
     TimeoutCheck(102);
}

// Wait for ACK to be received then send set gateway IP address command
void WiFiState103(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetGatewayAddr();                 // Set gateway address command
         WiFiStateFuncPtr = WiFiState104;      // Go wait for ACK
     }
     TimeoutCheck(103);
}

// Wait for ACK to be received then send set network mode command
void WiFiState104(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetNetworkMode();                 // Set network mode command
         WiFiStateFuncPtr = WiFiState105;      // Go wait for ACK
     }
     TimeoutCheck(104);
}

// Wait for ACK to be received then send set SSID command
void WiFiState105(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetCpSsid();                      // Set SSID command
         WiFiStateFuncPtr = WiFiState106;      // Go wait for ACK
     }
     TimeoutCheck(105);
}

// Wait for ACK to be received then send set regional domain command
void WiFiState106(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetRegionalDomain();              // Set regional domain command
         WiFiStateFuncPtr = WiFiState107;      // Go wait for ACK
     }
     TimeoutCheck(106);
}

// Wait for ACK to be received then send set channel list command
void WiFiState107(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetChannelList();                 // Set channel list command
         WiFiStateFuncPtr = WiFiState108;      // Go wait for ACK
     }
     TimeoutCheck(107);
}

// Wait for ACK to be received then send set channel list retry count command
void WiFiState108(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetListRetryCount();              // Set channel list command
         WiFiStateFuncPtr = WiFiState109;      // Go wait for ACK
     }
     TimeoutCheck(108);
}

#if USE_PASSWORD
// Wait for ACK to be received then send set security command (using password)
void WiFiState109(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetSecurityWpa();                 // Set security command with password
         WiFiStateFuncPtr = WiFiState110;      // Go wait for ACK
     }
     TimeoutCheck(109);
}
#else
// Wait for ACK to be received then send set security command (using 32 bit binary key)
void WiFiState109(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiSetSecurityWpaBinary();           // Set security command with binary key
         WiFiStateFuncPtr = WiFiState110;      // Go wait for ACK
     }
     TimeoutCheck(109);
}
#endif

// Wait for ACK to be received then send the connect command
void WiFiState110(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiConnect();                        // Issue connect command
         WiFiStateFuncPtr = WiFiState111;      // Go wait for ACK
     }
     TimeoutCheck(25);
}

// Wait for ACK to be received
void WiFiState111(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         WiFiEvents.ConStatChng = FALSE;       // Clear connection status changed flag
         BlinkTimer = get_timer();             // Get another timer handle
         set_timer(BlinkTimer,1);
#if USE_PASSWORD == TRUE
         set_timer(WiFiTimerHandle,60);        // Set timeout to 60 seconds
#else
         set_timer(WiFiTimerHandle,6);         // Set timeout to 6 seconds
#endif
         WiFiStateFuncPtr = WiFiState112;      // Go wait for connection status changed message
     }
     TimeoutCheck(111);
}

// Wait for connection status changed asynchronous message
void WiFiState112(void) {
     if (WiFiEvents.ConStatChng == TRUE) {
         free_timer(BlinkTimer);               // Release the timer handle to the pool
         if (ConnectStatus == 0x01) {
             WiFiEvents.IPAddrAss = FALSE;     // Clear IP address assigned flag
             set_timer(WiFiTimerHandle,6);     // Set timeout to 6 seconds
             WiFiStateFuncPtr = WiFiState113;  // Go wait for IP address assigned message
         }else{
             UpdateLabel19("Connection failed",CL_RED);
             WiFiStateFuncPtr = WiFiState0;    // Return to holding state
         }
     }

     if ((check_timer(BlinkTimer) == 0) && (Label19.Font_Color == CL_BLACK)) {
         set_timer(BlinkTimer,1);
         UpdateLabel19("Wait",CL_RED);
     }
     if ((check_timer(BlinkTimer) == 0) && (Label19.Font_Color == CL_RED)) {
         set_timer(BlinkTimer,1);
         UpdateLabel19("",CL_BLACK);
     }
     if (check_timer(WiFiTimerHandle) == 0) {
         free_timer(BlinkTimer);
#if USE_PASSWORD == TRUE
         UpdateLabel19("Timed out after 60 seconds",CL_RED);
#else
         UpdateLabel19("Timed out after 6 seconds",CL_RED);
#endif

         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
}

// Wait for IP address assigned asynchronous message
void WiFiState113(void) {
     char Message[40];

     if (WiFiEvents.IPAddrAss == TRUE) {
         sprintf(Message,"IP address: %u.%u.%u.%u",AssignedIP[0],AssignedIP[1],AssignedIP[2],AssignedIP[3]);
         if (strlen(Message) <= 40)
             UpdateLabel19(Message,CL_GREEN);
         else
             UpdateLabel19("IP message too long",CL_RED);
         EnableButton3();
         WiFiStateFuncPtr = WiFiState0;    // Return to holding state
     }
     TimeoutCheck(113);
}
//-------------------------------------------------------------------------------------------------


// Combined sequence to create UDP socket and start a UDP server **********************************
// This sequence is called when the Start USB button is pressed on the LCD.
// Issue socket create command
void WiFiState120(void) {
     WiFiEvents.SockCreaResp = FALSE;          // Clear socket create response flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiSocketCreate(UDP);                    // Issue socket create command
     WiFiStateFuncPtr = WiFiState121;          // Go wait for socket create response
}

// Wait for socket create response to be received
void WiFiState121(void) {
     if (WiFiEvents.SockCreaResp == TRUE) {    // Wait for socket create response flag
         if (SocketHandle < 254) {
             WiFiEvents.SockBindResp = FALSE;  // Clear socket bind response flag
             BindResult = 255;                 // Invalidate bind result
             set_timer(WiFiTimerHandle,4);     // Set timeout to 4 seconds
             WiFiSocketBind(SocketHandle,UDP_LocPort);  // Issue socket bind command
             WiFiStateFuncPtr = WiFiState122;
         }else{
             UpdateLabel19("Failed to create a socket",CL_RED);
             WiFiStateFuncPtr = WiFiState0;
         }
     }
     TimeoutCheck(121);
}

// Wait for socket bind response to be received
void WiFiState122(void) {
     if (WiFiEvents.SockBindResp == TRUE) {
         if (BindResult == 0) {
             UpdateLabel19("UDP server socket ready",CL_GREEN);
             EnableButton4();                  // Allow disconnect button
             WiFiEvents.SockRcvFromResp = FALSE; // Clear socket receive from response flag
             WiFiSocketReceiveFrom(SocketHandle,128); // Issue receive from command, data length
             WiFiStateFuncPtr = WiFiState123;  // Go wait for socket receive from response
         }else{
             UpdateLabel19("Bind failed",CL_RED);
         }
     }
     TimeoutCheck(122);
}

// Wait for socket receive from response to be received (no timeout)
void WiFiState123(void) {
     if (WiFiEvents.SockRcvFromResp == TRUE) {
         switch (UDP_Buf[0]) {
             case 'A':
                 strcpy(DataBuffer,"Command A received\r\n");
                 break;

             case 'B':
                 strcpy(DataBuffer,"Command B received\r\n");
                 break;

             default:
                 strcpy(DataBuffer,"Unimplemented command\r\n");
                 break;
         }

         WiFiEvents.SockSendToResp = FALSE;    // Clear socket send to response flag
         WiFiSocketSendTo(SocketHandle,&DataBuffer);
         WiFiStateFuncPtr = WiFiState124;      // Go wait for socket send to response
     }
}

// Wait for socket send to response to be received (no timeout)
void WiFiState124(void) {
     if (WiFiEvents.SockSendToResp == TRUE) {
         WiFiEvents.SockRcvFromResp = FALSE;   // Clear socket receive from response flag
         WiFiSocketReceiveFrom(SocketHandle,128); // Issue receive from command, data length
         WiFiStateFuncPtr = WiFiState123;       // Go wait for socket receive from response
     }
}
//-------------------------------------------------------------------------------------------------


// Combined sequence to close a socket and disconnect from access point ***************************
// This sequence is called when the Disconnect button is pressed on the LCD.
// Close the current socket
void WiFiState130(void) {
     ACK_Received = FALSE;                     // Clear ACK flag
     set_timer(WiFiTimerHandle,4);             // Set timeout to 4 seconds
     WiFiSocketClose(SocketHandle);            // Issue socket close command
     WiFiStateFuncPtr = WiFiState131;                                                                                                                                                                                                                                                 // Go wait for ACK

}

// Wait for ack
void WiFiState131(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         ACK_Received = FALSE;                 // Clear ACK flag
         set_timer(WiFiTimerHandle,4);         // Set timeout to 4 seconds
         WiFiDisconnect();                     // Issue disconnect command
         WiFiStateFuncPtr = WiFiState132;      // Go wait for ACK
     }
     TimeoutCheck(131);
}

// Wait for ack
void WiFiState132(void) {
     if (ACK_Received == TRUE) {               // Waiting for ACK
         WiFiEvents.IPAddrAss = FALSE;         // Clear connection status changed flag
         WiFiStateFuncPtr = WiFiState133;      // Go wait for connection status changed response
     }
     TimeoutCheck(132);
}

void WiFiState133(void) {
     if (WiFiEvents.IPAddrAss == TRUE) {
         EraseButton4();
         EraseButton3();
         EraseButton2();
         UpdateLabel19("Disconnected from access point",CL_BLUE);
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
     TimeoutCheck(133);
}
//-------------------------------------------------------------------------------------------------









// The Dreaded Evil Error Event *******************************************************************
void WiFiState666(void) {
/*
     char *BufNum;

     BufNum = GetBuffer();
     sprintf(BufNum,"%u\r\n",ErrorType[0]);
     USB_PutQueue(BufNum);
*/
     switch (ErrorType[0]) {
         case 60: USB_PutQueue("Baud rate generator error\r\n"); break;
         case 61: USB_PutQueue("Invalid connection profile ID error\r\n"); break;
         case 62: USB_PutQueue("Wi-Fi already connected error\r\n"); break;
         case 63: USB_PutQueue("Wi-Fi already disconnected error\r\n"); break;
         case 64: USB_PutQueue("Close socket failed error\r\n"); break;
         case 65: USB_PutQueue("Socket send to timeout error\r\n"); break;
         case 66: USB_PutQueue("Scan index out of range error\r\n"); break;
         case 67: USB_PutQueue("ICMP ping flood error\r\n"); break;
         case 68: USB_PutQueue("ICMP ping in use error\r\n"); break;
         case 69: USB_PutQueue("Socket reconfirm failed error\r\n"); break;
         case 71: USB_PutQueue("Serial transmit buffer allocation error\r\n"); break;
         case 72: USB_PutQueue("General assert error\r\n"); break;
         case 73: USB_PutQueue("Invalid power save mode error\r\n"); break;
         case 74: USB_PutQueue("Busy (hibernate mode) error\r\n"); break;
         case 75: USB_PutQueue("Busy (scan mode) error\r\n"); break;
         default: USB_PutQueue("Unknown error\r\n"); break;
     }
         WiFiEvents.Error = FALSE;             // Clear the error flag for this event
         //free_timer(WiFiTimerHandle);          // Release the timer handle to the pool
         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
}

// The timeout subroutine *************************************************************************
void TimeoutCheck(unsigned int TimeoutState) {
     char *BufNum;

     if (check_timer(WiFiTimerHandle) == 0) {  // If timeout

         BufNum = GetBuffer();
         sprintf(BufNum,"Timeout in state %u\r\n",TimeoutState);
         USB_PutQueue(BufNum);

         WiFiStateFuncPtr = WiFiState0;        // Return to holding state
     }
}